"""
    main: Handles the command-line interface to the tool
"""

import socket
import sys
import os

from .interface import DockerShield
from .arguments import parse_argv

def main(argv):
    """
    Run with command line arguments
    NOTE: If not passing sys.argv, remember to include equivalent of sys.argv[0] even though we ignore it
    """
    while True:
        args = parse_argv(argv)
        with DockerShield(args) as shield:
            shield.run()
        if not shield.restart_requested:
            break



import threading
import select
import logging
import time

from .parser import HttpParser, Message, HTTP_400_RESPONSE

class Connection(threading.Thread):
    """
        Class to represent a two-way connection
        This acts as a Man-In-The-Middle between docker and docker clients
        Each connection runs in its own thread.
    """

    DENIED_RESPONSE = bytes(HTTP_400_RESPONSE,"utf8")
    CHUNK_SIZE=1024

    def __init__(self, client_socket, upstream_socket, parent, filters):
        """
        Constructor Method
            Inputs:
                client_socket: Socket connected to the docker client.
                upstream_socket: Socket connected to the docker server
                parent: A DockerShield instance which this connection is a part of
                filters: List of filtering rules
        """
        self.filtered = client_socket
        self.upstream = upstream_socket
        self.socket_map = {
            self.filtered: self.upstream,
            self.upstream: self.filtered
        }
        self.running = False
        self.lock = threading.RLock()
        self.parent = parent
        self.filters = filters
        self.parser_map = {
            self.filtered: HttpParser(),
            self.upstream: HttpParser()
        }
        self.last_messages = {
            self.filtered: [],
            self.upstream: []
        }
        self.closed_map = {}
        self.upgraded = False
        self.strikes = 0
        super(Connection, self).__init__(daemon=True)

    def close(self):
        """
        Close both ends of the connection
        Also sets flag so that the thread will stop.
        """
        with self.lock:
            self.running = False
            if self.filtered is not None:
                self.filtered.close()
                self.filtered = None
            if self.upstream is not None:
                self.upstream.close()
                self.upstream = None
            if self in self.parent.connections:
                self.parent.connections.remove(self)

    def handle_closed(self, closed_socket):
        """
        Handle a socket closed event
            Inputs:
                closed_socket: The socket that was just closed.
                Must be one of [self.upstream, self.filtered]
        """

        # Check if the socket was already closed before
        if not self.closed_map.get(closed_socket, False):
            # Print information only if the socket wasn't already closed before
            logging.info("Connection closed on %s end" % self.describe_socket(closed_socket))
            self.closed_map[closed_socket] = True

        # We only stop running if:
        #   1. The client disconnects during a regular HTTP connection
        #   2. The server disconnects at any point
        # We don't stop running if:
        #   1. The client disconnects, but connection is upgraded and server is still connected
        if (closed_socket is self.filtered and not self.upgraded) \
            or closed_socket is self.upstream:
            with self.lock:
                self.running = False

    def read_socket_data(self, timeout=30.0):
        """
        Reads data from both ends of the connection,
        applies filtering rules, and if allowed, passes it to the other end of the connection.
        This also detects if either end of the connection is closed.
        """

        start_time = time.time()
        # We know that poll is more scalable than select, but select is easier (plus we only have 2 sockets, right?)
        readable, _, _ = select.select([self.filtered, self.upstream], [self.filtered, self.upstream], [], timeout)

        # When there is no data, print a warning every 10 seconds
        if len(readable) == 0 and (time.time() - start_time) >= 10.0:
            logging.warn("No data on either end.")

        for input_socket in readable:
            data = input_socket.recv(self.CHUNK_SIZE)
            if len(data) == 0:
                self.handle_closed(input_socket)
                continue

            logging.debug("Read %d bytes from %s" % (len(data), self.describe_socket(input_socket)))
            if not self.upgraded:
                self.parser_map[input_socket].feed(data)
            else:
                self.socket_map[input_socket].sendall(data)

    def describe_socket(self, socket_instance):
        """
        Print a string that describes a socket
            Inputs:
                socket_instance: The socket. Must be one of [self.filtered, self.upstream]
        """
        if socket_instance is self.filtered:
            return "<client>"
        elif socket_instance is self.upstream:
            return "<docker>"
        else:
            assert(socket_instance in [self.filtered, self.upstream])
            return "<unknown>"


    def handle_http_messages(self):
        """
        Handle a completed HTTP message (either a request or response)
        """
        for input_socket, parser in self.parser_map.items():
            if not parser.has_message():
                continue
            output_socket = self.socket_map[input_socket]
            message = parser.pop_message()
            if not self.filters.is_message_allowed(message):
                logging.info("Filtered out a message: %s from %s" % (message.describe(), self.describe_socket(input_socket)))
                if input_socket is self.filtered:
                    input_socket.sendall(self.DENIED_RESPONSE)
                    input_socket.close()
                    with self.lock:
                        self.running = False
                continue

            logging.debug("Allowed a message: %s from %s" % (message.describe(), self.describe_socket(input_socket)))

            if message.type in ["response"]:
                assert(input_socket is self.upstream)
                if message.status in [101] and message.status_text in ["UPGRADED"]:
                    logging.warn("The connection was upgraded; no more filtering will be applied!")
                    self.upgraded = True
            output_socket.sendall(message.raw)

    def inner_loop(self,timeout=120.0):
        """
        The inner loop for the thread
        """
        self.read_socket_data(timeout)
        if not self.upgraded:
            self.handle_http_messages()

    def run(self):
        """
        Thread task.
        Loops until self.running is False
        """
        logging.debug("Started")
        with self.lock:
            self.running = True

        running = True
        while running:
            try:
                self.inner_loop()
            except OSError as e:
                logging.warn("Connection broken: %s" % repr(e))
                running = False

            with self.lock:
                running = running and self.running
        self.close()



import logging
import re
from urllib.parse import urlparse

HTTP_400_RESPONSE = "HTTP/1.1 400 Bad Request\r\nContent-Type: text/plain; charset=utf-8\r\n\r\n"


class Message():
    def __init__(self, raw_bytes):
        self.raw = raw_bytes
        self.string = str(self.raw, 'utf8')
        lines = self.string.split("\r\n")
        self.parse_header(lines.pop(0))
        self.parse_fields(lines)
        self.content_length = int(self.fields.get("Content-Length", 0))
        self.content = bytes()

    def parse_fields(self, lines):
        self.fields = {}
        for line in lines:
            for match in re.finditer(r'(?P<key>^.+): (?P<value>.+)$', line):
                self.fields[match.group("key")] = match.group("value")


    def parse_header(self, header):
        if header.startswith("HTTP/"):
            self.type = "response"
            for match in re.finditer(r'^HTTP/\d+\.\d+\s+(?P<status>\d+)\s+(?P<status_text>.*)$', header):
                self.status = int(match.group("status"))
                self.status_text = match.group("status_text")
        else:
            self.type = "unknown"
            for match in re.finditer(r'^(?P<method>\S+)\s+(?P<url>.+)\s+(HTTP/\d+(\.\d+)?)?$', header):
                self.method = match.group("method")
                self.url = match.group("url")
                self.type = "request"
                self.url_path = urlparse(self.url).path

    def finished(self):
        return len(self.content) == self.content_length

    def add_content(self, data):
        self.content += data
        self.raw += data

    def describe(self):
        if self.type in ["request"]:
            return "Request: %s %s" % (self.method, self.url_path)
        elif self.type in ["response"]:
            return "Response: %d %s, %d bytes" % (self.status, self.status_text, self.content_length)
        else:
            return "<unknown message %d bytes, first 50 bytes: %.50s>" % (len(self.raw), repr(str(self.raw, 'utf8')))


class HttpParser():
    DELIMITERS = [
        bytes("\r\n\r\n", 'utf8')
    ]

    class State():
        WAITING_FOR_HTTP = 0
        WAITING_FOR_CONTENT = 1


    def __init__(self):
        self.messages = []
        self.raw_message_header = bytes()
        self.next_message = None
        self.state = HttpParser.State.WAITING_FOR_HTTP


    def feed_http_header_one_byte(self, data):
        assert(isinstance(data, bytes))
        #logging.debug("State: %d" % self.state)
        assert(self.state is HttpParser.State.WAITING_FOR_HTTP)
        assert(self.next_message is None)
        self.raw_message_header += data
        #logging.debug("\n"+repr(str(self.raw_message_header, 'utf8')))
        if any(
            self.raw_message_header.endswith(delimiter) \
                for delimiter in self.DELIMITERS
        ):
            self.next_message = Message(self.raw_message_header)
            self.raw_message_header = bytes()
            if not self.next_message.finished():
                self.state = HttpParser.State.WAITING_FOR_CONTENT
                logging.debug("Waiting for message content (%s)." % (self.next_message.describe(),))
            else:
                self.messages += [self.next_message]
                self.next_message = None
                logging.debug("Completed message with no content.")

    def feed_http_header(self, data):
        for single_byte in data:
            self.feed_http_header_one_byte(bytes([single_byte]))

    def feed_content(self, data):
        assert(self.state is HttpParser.State.WAITING_FOR_CONTENT)
        assert(isinstance(self.next_message, Message))
        self.next_message.add_content(data)
        if self.next_message.finished():
            self.messages += [self.next_message]
            self.next_message = None
            self.state = HttpParser.State.WAITING_FOR_HTTP
            logging.debug("Completed content.")

    def feed(self, data):
        index = 0
        while self.state is HttpParser.State.WAITING_FOR_HTTP \
            and index < len(data):
            single_byte = data[index:index+1]
            self.feed_http_header_one_byte(single_byte)
            index += 1

        if self.state is self.State.WAITING_FOR_CONTENT:
            logging.debug("State: %d, index %d out of %d" % (self.state, index, len(data)))
            self.feed_content(data[index:])

    def has_message(self):
        return len(self.messages) > 0

    def pop_message(self):
        return self.messages.pop(0)

    def peek_message(self):
        return self.messages[0]

    def generate_messages(self, bytes_like):
        for data in bytes_like:
            self.feed(data)
            while self.has_message():
                yield self.pop_message()



from argparse import Namespace

import socket
import sys
import os
import logging
from shutil import copy2
from subprocess import check_call, check_output, CalledProcessError
import time
import select

from .connection import Connection
from .arguments import config, dump_config, SYSTEMD_PREINSTALL_FILE, SYSTEMD_POSTINSTALL_FILE, CONFIGURATION_FILE
from .systemd import systemd
from .filtering import Filters

LOG_FORMATTER = "%(asctime)s : %(levelname)s : %(message)s"

class DockerShield():
    """
    Main class.
    Typically this will be a singleton.
    """

    def __init__(self, args):
        """
        Constructor
            Input:
                args: An argparse.Namespace, see arguments.py for arguments definition
        """
        self.args = args
        self.filtered_socket = None
        self.connections = []
        assert(isinstance(args, Namespace))

    class Exception(Exception):
        """
        Exception wrapper
        """
        pass

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        self.close()

    def close(self):
        """
        Close all active connections and stop listening for clients
        """
        i = 0
        while len(self.connections) > 0:
            logging.debug("Close connection number %d" % (i,))
            connection = self.connections.pop()
            connection.close()
            i += 1

        if self.filtered_socket is not None:
            logging.debug("Close filtered socket at %s" % (self.args.filtered_socket_path))
            self.filtered_socket.close()
            self.filtered_socket = None

        if os.path.exists(self.args.filtered_socket_path):
            logging.debug("Unlink filtered socket path at %s" % (self.args.filtered_socket_path,))
            os.unlink(self.args.filtered_socket_path)

    def connect_upstream(self):
        """
        Open a socket to the docker server
        """
        upstream_socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        upstream_socket.connect(self.args.docker_socket_path)
        return upstream_socket

    def open(self):
        """
        Open the downstream socket for connections.
        Also does a test that the upstream socket exists and can be connected to.
        Handles some common errors and raises exceptions.
        """
        assert(self.filtered_socket is None)
        if not os.path.exists(self.args.docker_socket_path):
            raise DockerShield.Exception("docker_socket_path \"%s\" does not exist" % (self.args.docker_socket_path,))

        logging.debug("Test upstream socket at %s actually exists and accepts connections" % (self.args.docker_socket_path,))
        upstream_socket = self.connect_upstream()
        upstream_socket.close()

        logging.debug("Create filtered socket at %s" % (self.args.filtered_socket_path,))
        if os.path.isdir(self.args.filtered_socket_path):
            logging.warn("The filtered socket at %s is a directory; this could happen if you tried to mount the path as a volume before running this script." \
                % (self.args.filtered_socket_path,)
            )
            if not os.listdir(self.args.filtered_socket_path):
                logging.warn("Since the directory is empty, we will try to remove it")
                os.rmdir(self.args.filtered_socket_path)
            else:
                raise DockerShield.Exception("A non-empty directory exists at %s" % (self.args.filtered_socket_path,))

        if os.path.exists(self.args.filtered_socket_path):
            raise DockerShield.Exception("filtered_socket_path \"%s\" already exists" % (self.args.filtered_socket_path,))

        self.filtered_socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        logging.debug("Listen for clients at %s" % (self.args.filtered_socket_path,))
        self.filtered_socket.bind(self.args.filtered_socket_path)
        self.filtered_socket.listen(1)

    def configure_logging(self):
        """
        Configure the Python built-in logging module
        """
        # Verbose means to log everything,
        # Silent means to log nothing,
        # Otherwise, we log at level INFO
        if self.args.verbose:
            assert(not self.args.silent)
            logging.basicConfig(format=LOG_FORMATTER, level=logging.DEBUG)
        elif not self.args.silent:
            assert(not self.args.verbose)
            logging.basicConfig(format=LOG_FORMATTER, level=logging.INFO)

    def configure_filters(self):
        filters_as_dictionary = {
            "method" : {
                "blacklist" : self.args.blacklist_method,
                "whitelist" : self.args.whitelist_method
            },
            "url" : {
                "include_version" : self.args.filter_includes_version,
                "blacklist" : self.args.blacklist,
                "whitelist" : self.args.whitelist
            }
        }
        self.filters = Filters(filters_as_dictionary)

    def file_has_changed(self, filename):
        return os.stat(filename).st_mtime >= self.start_time

    def accept_connection(self):
        """
        Accept a connection, spawn Connection instance and add to table
        """
        logging.debug("Waiting on connection...")
        connection, _ = self.filtered_socket.accept()
        connection_id = len(self.connections)
        logging.debug("Client[%d] connected to %s" % (connection_id, self.args.filtered_socket_path,))
        new_connection = Connection(connection, self.connect_upstream(), self, self.filters)
        self.connections += [new_connection]
        # The Connection is a Thread, so must be started
        new_connection.start()

    def inner_loop(self):
        if not self.args.watch_config:
            self.accept_connection()
            return

        if self.file_has_changed(CONFIGURATION_FILE):
            logging.info("Configuration file %s was modified, stopping..." % CONFIGURATION_FILE)
            self.restart_requested = True
            self.running = False
            return

        readable, _, _ = select.select([self.filtered_socket], [], [], 2.0)
        if self.filtered_socket in readable:
            self.accept_connection()



    def run(self):
        """
        Main loop
        """
        self.configure_logging()
        logging.info("Configuration is at "+CONFIGURATION_FILE)
        logging.debug("Full Configuration Dump\n"+dump_config())

        if systemd.process_args(self.args):
            return

        if self.args.dry_run:
            # FIXME: Is it really a dry run if it has side effects?
            logging.info("Dry run. Will check that sockets can be opened and then exit")

        self.configure_filters()

        self.open()
        self.running = not self.args.dry_run
        self.restart_requested = False
        self.start_time = time.time()
        while self.running:
            try:
                self.inner_loop()
            except KeyboardInterrupt:
                logging.info("Got KeyboardInterrupt, exiting...")
                self.running = False

        self.close()
        if self.args.dry_run:
            logging.info("Dry run. Success!")
        logging.debug("Clean exit.")



"""
    This module contains command line argument and configuration file parsing.
    FIXME: The name arguments.py might be confusing since it also has configuration file?
"""

from argparse import ArgumentParser, Namespace, ArgumentDefaultsHelpFormatter

import os
import sys

# PyYAML
from yaml import load, dump

# As recommended, try to use the C YAML parser if it exists
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

CONFIGURATION_FILE = os.path.realpath(os.path.join(os.path.dirname(__file__), "config", "dockershield.yml"))

from .systemd.systemd import SYSTEMD_POSTINSTALL_FILE, SYSTEMD_PREINSTALL_FILE

global config

def get_config(config_file=CONFIGURATION_FILE):
    with open(config_file, "r") as f:
        config = load(f, Loader=Loader)
    return config

def dump_config(config_object=None):
    global config
    if config_object is None:
        config_object = config
    output = dump(config_object, Dumper=Dumper)
    return output


def get_argparser(argv, parents=None, conflict_handler="resolve"):
    """
    Returns argparse compatible parser
    NOTE: This tool should always run successfully with no arguments.
            Default values are parsed from the CONFIGURATION_FILE

        Inputs:
            argv: Currently ignored. Might be used to patch argparse functionality.
            parents: Parent argparse Parsers. Use at own risk.
            conflict_handler: How to resolve conflicts with parent parsers. Use at own risk.
        Outputs:
            argparse.ArgumentParser instance
    """
    if parents is None:
        parents = []

    parser = ArgumentParser(
        description="Wraps the docker unix socket with some filtering to deny non-whitelisted commands"
        ". You may find it more convenient to edit the values in %s" % (CONFIGURATION_FILE,),
        parents=parents,
        conflict_handler=conflict_handler,
        # Use this class to show the default values. By default argparse does not show default.
        formatter_class=ArgumentDefaultsHelpFormatter
    )

    # Self documenting arguments?
    global config
    config = get_config()

    parser.add_argument(
        "--blacklist",
        type=str,
        help="URL regexes to blacklist",
        nargs="*",
        default=config.get("filters", {}).get("url", {}).get("blacklist", [])
    )
    parser.add_argument(
        "--whitelist",
        type=str,
        help="URL regexes to whitelist. NOTE: Only whitelists if the blacklist did not match.",
        nargs="*",
        default=config.get("filters", {}).get("url", {}).get("whitelist", [])
    )


    parser.add_argument(
        "--docker-socket-path",
        type=str,
        help="Path to the docker unix socket.",
        default=os.path.realpath(config.get("docker_socket_path","/var/run/docker.sock"))
    )

    parser.add_argument(
        "--filtered-socket-path",
        type=str,
        help="Path to the sanitised socket, which clients can connect to.",
        default=os.path.realpath(config.get("filtered_socket_path", "dockershield.sock"))
    )

    parser.add_argument(
        "--blacklist-method",
        type=str,
        help="Methods to blacklist",
        nargs="*",
        default=config.get("filters", {}).get("method", {}).get("blacklist", [])
    )
    parser.add_argument(
        "--whitelist-method",
        type=str,
        help="Methods to whitelist. NOTE: Only whitelists if the blacklist did not match.",
        nargs="*",
        default=config.get("filters", {}).get("method", {}).get("whitelist", [])
    )

    parser.add_argument(
        "--filter-includes-version",
        action="store_true",
        help="Does the URL filtering include the API version string?",
        default=config.get("filters", {}).get("url", {}).get("include_version")
    )

    parser.add_argument(
        "--dry-run",
        action="store_true",
        help="Verifies that the script could run, but stops immediately."
    )

    parser.add_argument(
        "--verbose",
        action="store_true",
        help="Prints all levels of logging messages to stdout.",
        default=config.get("verbose", False)
    )
    parser.add_argument(
        "--silent",
        action="store_true",
        help="Hide all logging messages, including warnings and errors. Incompatible with --verbose",
        default=config.get("silent", False)
    )

    parser.add_argument(
        "--watch_config",
        action="store_true",
        help="If set, will periodically poll the config file at %s for changes and restart on a change.",
        default=config.get("watch_config", False)
    )

    parser.add_argument(
        "--systemd-install",
        action="store_true",
        help="Installs this tool as a systemd service. The unit file is copied from %s to %s."
            % (SYSTEMD_PREINSTALL_FILE, SYSTEMD_POSTINSTALL_FILE)
    )
    parser.add_argument(
        "--systemd-uninstall",
        action="store_true",
        help="DANGER: Removes any installed systemd service at %s." % (SYSTEMD_POSTINSTALL_FILE,)
    )

    return parser


def parse_argv(argv):
    """
    Parses command line arguments in argv
    NOTE: argv is assumed to include a excecutable name, so argv[0] is ignored
    """
    parser = get_argparser(argv)
    # argparse parsers ignore argv[0]
    return parser.parse_args(argv[1:])

config = get_config()



import re
import logging
from .parser import Message

class Filters():

    class Exception(Exception):
        pass

    def __init__(self, d):
        self.blacklisted_methods = d.get("method", {}).get("blacklist", []) or []
        self.whitelisted_methods = d.get("method", {}).get("whitelist", []) or []

        blacklist_url_strings = d.get("url", {}).get("blacklist", []) or []
        whitelist_url_strings = d.get("url", {}).get("whitelist", []) or []

        self.blacklist_regex = [re.compile('^'+s+'$') for s in blacklist_url_strings]
        self.whitelist_regex = [re.compile('^'+s+'$') for s in whitelist_url_strings]

        self.include_version = d.get("url", {}).get("include_version", False)
        self.strict_blacklist = d.get("url", {}).get("strict_blacklist", False)

        self.version_regex = re.compile(r'^v\d+\.\d+$')
        self.has_any_blacklist = any((
            self.blacklisted_methods,
            self.blacklist_regex
        ))

    def is_listed(self, comparison_string, regex_list, logging_label):
        for regex in regex_list:
            logging.debug("%s: Check %s vs %s" % (logging_label, comparison_string, regex))
            if regex.search(comparison_string):
                return True
        return False

    def whitelisted(self, comparison_string):
        return self.is_listed(comparison_string, self.whitelist_regex, "Whitelist")

    def blacklisted(self, comparison_string):
        return self.is_listed(comparison_string, self.blacklist_regex, "Blacklist")

    def is_request_allowed(self, message):
        assert(message.type in ["request"])
        endpoint_list = message.url_path.split("/")
        if endpoint_list.pop(0) != "":
            logging.debug("Block URL that doesn't start with /")
            return False

        # Filter method
        if message.method in self.blacklisted_methods \
            and message.method not in self.whitelisted_methods:
            return False

        version_string = endpoint_list[0]
        if not self.version_regex.search(version_string):
            logging.warn("No version string.")
        elif not self.include_version:
            endpoint_list.pop(0)

        comparison_string = "/".join(endpoint_list)
        logging.debug("Check %s" % comparison_string)

        if self.strict_blacklist:
            return not self.blacklisted(comparison_string)
        else:
            return self.whitelisted(comparison_string) or not self.blacklisted(comparison_string)

    def is_response_allowed(self, message):
        assert(message.type in ["response"])
        return True

    def is_message_allowed(self, message):
        assert(isinstance(message, Message))
        if not self.has_any_blacklist:
            logging.warn("No blacklisting enabled; all messages allowed!")
        if message.type in ["request"]:
            return self.is_request_allowed(message)
        elif message.type in ["response"]:
            return self.is_response_allowed(message)
        else:
            logging.warn("Allowing unknown message %s" % message.describe())
        return True



"""
    This is the entry point when you run python on this module as a directory
"""

import sys
import os

if __name__ == "__main__":
    # Need to patch the sys.path variable in order to deal with Python relative imports properly
    sys.path.insert(0, os.path.realpath(os.path.join(os.path.dirname(__file__), "..")))
    from dockershield.main import main
    sys.exit(main(sys.argv))



import os
from setuptools import setup
from setuptools.command.install import install
from sys import executable
from ast import literal_eval

# NOTE: setuptools doesn't provide an uninstall script :/
class PostInstallCommand(install):
    def run(self):
        # Run the script to install itself as a systemd unit
        if literal_eval(os.environ.get("SYSTEMD_INSTALL", "False")):
            from subprocess import check_call
            check_call([executable, "dockershield", "--systemd-install"])
        install.run(self)

# Use the requirements.txt
with open("requirements.txt", "r") as f:
    requirements = f.readlines()

# setuptools compatible interface.
setup(
    name="dockershield",
    version="0.0.1",
    author="Sam Moore",
    author_email="smoore@elementengineering.com.au",
    description=(
        "Interface to protect your docker socket, so that you can pass it to containers but prevent them running dangerous commands"
    ),
    license="BSD",
    keywords="docker",
    url="",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License"
    ],
    scripts=["scripts/dockershield"],
    packages=["dockershield"],
    install_requires=[
        # Read directly from requirements.txt
        requirements
    ],
    # Include all files in MANIFEST.in
    include_package_data=True,
    # For post installation scripts
    cmdclass={
        "install" : PostInstallCommand
    }
)



"""
    An interface to quickly install this tool as a systemd service
"""

import logging
import os
from shutil import copy2
from subprocess import check_call, check_output, CalledProcessError
import time

SYSTEMD_PREINSTALL_FILE = os.path.realpath(os.path.join(os.path.dirname(__file__), "dockershield.service"))
SYSTEMD_POSTINSTALL_FILE = "/usr/lib/systemd/system/dockershield.service"

def stop():
    logging.debug("Stop any running service")
    check_call(["systemctl", "stop", "dockershield"])

def reload_daemon():
    logging.debug("Reload systemd daemon")
    check_call(["systemctl", "daemon-reload"])

def enable():
    logging.debug("Enable systemd service")
    check_call(["systemctl", "enable", "dockershield"])

def status():
    output = check_output(["systemctl", "status", "dockershield"])
    logging.debug("systemctl status: \n"+str(output, 'utf8'))
    return output

def restart():
    logging.debug("Start systemd service")
    check_call(["systemctl", "restart", "dockershield"])
    # FIXME: Hack: Wait for service to startup properly
    trial = 0
    max_trials = 3
    while trial < max_trials:
        trial += 1
        try:
            status()
            break
        except CalledProcessError:
            if trial > 3:
                raise
            time.sleep(10)

def disable():
    logging.debug("Disable systemd service")
    check_call(["systemctl", "disable", "dockershield"])


def install():
    os.environ["SYSTEMD_PAGER"] = ""
    logging.debug("Unit file at %s" % (SYSTEMD_PREINSTALL_FILE,))
    logging.debug("Destination %s" %  SYSTEMD_POSTINSTALL_FILE)
    logging.debug("Copy unit file")
    copy2(SYSTEMD_PREINSTALL_FILE, SYSTEMD_POSTINSTALL_FILE)
    reload_daemon()
    restart()

def uninstall():
    os.environ["SYSTEMD_PAGER"] = ""
    stop()
    disable()
    logging.debug("Remove unit file at %s" % SYSTEMD_POSTINSTALL_FILE)
    os.unlink(SYSTEMD_POSTINSTALL_FILE)
    reload_daemon()


def process_args(args):
    """
    Processes an argparse.Namespace and returns True if any actions were taken
    """
    result = False
    if args.systemd_install:
        logging.info("Installing as Systemd unit.")
        install()
        logging.info("Installation complete!")
        result = True
    if args.systemd_uninstall:
        logging.info("Removing any Systemd unit.")
        uninstall()
        logging.info("Removal complete!")
        result = True
    return result
